==> pg-transactional-outbox-0.1.0.tgz: package/src/outbox.spec.ts <==
/* eslint-disable @typescript-eslint/no-non-null-assertion */
import inspector from 'inspector';
import { OutboxServiceConfig } from './outbox-service';
import { disableLogger } from './logger';
import { Client, Pool } from 'pg';
import { initializeOutboxMessageStorage } from './outbox';

const isDebugMode = (): boolean => inspector.url() !== undefined;
if (isDebugMode()) {
  jest.setTimeout(600_000);
} else {
  disableLogger(); // Hide logs if the tests are not run in debug mode
  jest.setTimeout(7_000);
}

const config: OutboxServiceConfig = {
  pgReplicationConfig: {
    host: 'test_host',
    port: 5432,
    database: 'test_db',
    user: 'test_outbox_user',
    password: 'test_outbox_user_password',
  },
  settings: {
    dbSchema: 'test_schema',
    dbTable: 'test_table',
    postgresPub: 'test_pub',
    postgresSlot: 'test_slot',
  },
};

// required for the initializeOutboxMessageStorage
jest.mock('pg', () => {
  return {
    ...jest.requireActual('pg'),
    Pool: jest.fn().mockImplementation(() => ({
      connect: jest.fn(() => new Client()),
      on: jest.fn(),
      end: jest.fn(() => Promise.resolve()),
      removeAllListeners: jest.fn(),
    })),
    Client: jest.fn().mockImplementation(() => ({
      query: jest.fn(async (sql: string, _params: [any]) => {
        if (
          sql.indexOf(
            `INSERT INTO ${config.settings.dbSchema}.${config.settings.dbTable}`,
          )
        ) {
          return { rowCount: 1 };
        }
        throw new Error(`Missed to mock the following SQL query: ${sql}`);
      }),
      on: jest.fn(),
      release: jest.fn(),
    })),
  };
});

describe('Outbox unit tests', () => {
  describe('initializeOutboxMessageStorage', () => {
    it('should store the outbox message data to the database', async () => {
      // Arrange
      const pool = new Pool();
      const client = await pool.connect();
      client.query = jest
        .fn()
        .mockReturnValueOnce({
          rowCount: 1,
          rows: [{ created_at: new Date() }],
        })
        .mockReturnValueOnce({});
      const aggregateId = 'test-aggregate-id';
      const aggregateType = 'movie';
      const eventType = 'created';
      const payload = { test: 'data' };
      const storeOutboxMessage = initializeOutboxMessageStorage(
        aggregateType,
        eventType,
        config,
      );

      // Act
      const result = await storeOutboxMessage(aggregateId, payload, client);

      // Assert
      expect(client.query).toHaveBeenCalledTimes(2);
      expect(client.query).toHaveBeenCalledWith(
        expect.stringContaining(
          `INSERT INTO ${config.settings.dbSchema}.${config.settings.dbTable}`,
        ),
        [expect.any(String), aggregateType, aggregateId, eventType, payload],
      );
      expect(result).toEqual({
        id: expect.any(String),
        aggregateType,
        aggregateId,
        eventType,
        payload,
        createdAt: expect.any(Date),
      });
    });

    it('should throw an error if the outbox message could not be created', async () => {
      // Arrange
      const pool = new Pool();
      const client = await pool.connect();
      client.query = jest.fn().mockReturnValue({ rowCount: 0 });
      const storeOutboxMessage = initializeOutboxMessageStorage(
        'movie',
        'created',
        config,
      );

      // Act + Assert
      await expect(
        storeOutboxMessage('test-aggregate-id', { test: 'data' }, client),
      ).rejects.toThrow('Could not insert the message into the outbox!');
    });
  });
});

==> pg-transactional-outbox-0.1.6.tgz: package/src/outbox.spec.ts <==
/* eslint-disable @typescript-eslint/no-non-null-assertion */
import inspector from 'inspector';
import { OutboxServiceConfig } from './outbox-service';
import { disableLogger } from './logger';
import { Client, Pool } from 'pg';
import { initializeOutboxMessageStorage } from './outbox';

const isDebugMode = (): boolean => inspector.url() !== undefined;
if (isDebugMode()) {
  jest.setTimeout(600_000);
} else {
  disableLogger(); // Hide logs if the tests are not run in debug mode
  jest.setTimeout(7_000);
}

const config: OutboxServiceConfig = {
  pgReplicationConfig: {
    host: 'test_host',
    port: 5432,
    database: 'test_db',
    user: 'test_outbox_user',
    password: 'test_outbox_user_password',
  },
  settings: {
    dbSchema: 'test_schema',
    dbTable: 'test_table',
    postgresPub: 'test_pub',
    postgresSlot: 'test_slot',
  },
};

// required for the initializeOutboxMessageStorage
jest.mock('pg', () => {
  return {
    ...jest.requireActual('pg'),
    Pool: jest.fn().mockImplementation(() => ({
      connect: jest.fn(() => new Client()),
      on: jest.fn(),
      end: jest.fn(() => Promise.resolve()),
      removeAllListeners: jest.fn(),
    })),
    Client: jest.fn().mockImplementation(() => ({
      query: jest.fn(async (sql: string, _params: [unknown]) => {
        if (
          sql.indexOf(
            `INSERT INTO ${config.settings.dbSchema}.${config.settings.dbTable}`,
          )
        ) {
          return { rowCount: 1 };
        }
        throw new Error(`Missed to mock the following SQL query: ${sql}`);
      }),
      on: jest.fn(),
      release: jest.fn(),
    })),
  };
});

describe('Outbox unit tests', () => {
  describe('initializeOutboxMessageStorage', () => {
    it('should store the outbox message data to the database', async () => {
      // Arrange
      const pool = new Pool();
      const client = await pool.connect();
      client.query = jest
        .fn()
        .mockReturnValueOnce({
          rowCount: 1,
          rows: [{ created_at: new Date() }],
        })
        .mockReturnValueOnce({});
      const aggregateId = 'test-aggregate-id';
      const aggregateType = 'movie';
      const eventType = 'created';
      const payload = { test: 'data' };
      const storeOutboxMessage = initializeOutboxMessageStorage(
        aggregateType,
        eventType,
        config,
      );

      // Act
      const result = await storeOutboxMessage(aggregateId, payload, client);

      // Assert
      expect(client.query).toHaveBeenCalledTimes(2);
      expect(client.query).toHaveBeenCalledWith(
        expect.stringContaining(
          `INSERT INTO ${config.settings.dbSchema}.${config.settings.dbTable}`,
        ),
        [expect.any(String), aggregateType, aggregateId, eventType, payload],
      );
      expect(result).toEqual({
        id: expect.any(String),
        aggregateType,
        aggregateId,
        eventType,
        payload,
        createdAt: expect.any(Date),
      });
    });

    it('should throw an error if the outbox message could not be created', async () => {
      // Arrange
      const pool = new Pool();
      const client = await pool.connect();
      client.query = jest.fn().mockReturnValue({ rowCount: 0 });
      const storeOutboxMessage = initializeOutboxMessageStorage(
        'movie',
        'created',
        config,
      );

      // Act + Assert
      await expect(
        storeOutboxMessage('test-aggregate-id', { test: 'data' }, client),
      ).rejects.toThrow('Could not insert the message into the outbox!');
    });
  });
});
